/*
 * AccumulatingAction.java
 *
 * Created on November 1, 2005, 9:42 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package com.sunlabs.archsim.fleet.ships.accumulator;

import com.sunlabs.archsim.gasp.BranchAction;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.Payload;

/**
 *
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 * 
 *
 * @author mh193489
 */
class AccumulatingAction extends BranchAction {
        /* some constants */
        public static final int BRANCH_ACK= 0;
        public static final int BRANCH_OUT= 1;
    
    
        int sum = 0;
        boolean overflow= false;
        boolean underflow= false;
        
        /*
         * communicate the elected branch between selectBranch() and copyData()
         * so that in one case data and  in other a token can be poroduced
         */
        int selectedBranch; 
        
        public AccumulatingAction(String name) {
                super(name, 2);
        } // 

        public int selectBranch(Object data) {
                if (data != null) {
                        Data dataIn= ((Payload)data).data;

                        // check if data is valid or OOB
                        if (dataIn.valid) {
                                // acknowledge the data, 
                                selectedBranch= BRANCH_ACK;
                         } else {
                                // OOB data, i.e. end of sequence, thus output sum and reset
                                selectedBranch= BRANCH_OUT;
                        }
                        return selectedBranch;
                        
                } else {
                        director.fatalError("AccumulatorInAction.selectBranch(): data==null");
                        director.stop();			
                        return -1;
                }
                
        }        
        
        public Object copyData(Object data) {
                Data dataOut= null;
                Data dataIn= ((Payload)data).data;
                
                switch (selectedBranch ) {
                        case BRANCH_ACK:    
                                // acknowledge data
                                dataOut= new Data();
                                // valid data, hence extract the data from Payload and increment sum 
                                if (!overflow && !underflow) {
                                        sum= sum + (Integer)dataIn.value;
                                }
                                if (sum >= AccumulatorDefaults.OVERFLOW) {
                                        overflow= true;
                                        sum= AccumulatorDefaults.PLUS_INFINITY;                                        
                                }
                                if (sum <= AccumulatorDefaults.UNDERFLOW) {
                                        underflow= true;
                                        sum= AccumulatorDefaults.MINUS_INFINITY;                                        
                                }
                                break;
                        case BRANCH_OUT:    
                                // output the sum, if no overflow or underflow, otherwise output OOB
                                // TODO: -Infinity and +Infinity
                                if (overflow || underflow) {
                                        dataOut= new Data(sum, false);
                                } else {
                                        dataOut= new Data(sum);
                                }
                                // reset the accumulator
                                sum= 0;
                                overflow= false;
                                underflow= false;
                                
                                break;
                        default:            director.fatalError("AccumulatorInAction.copyData(): invalid branch "
                                                                    + selectedBranch);
                                            director.stop();
                }
                
                return dataOut;
        }
}

/*
                                    // log to journal
                                    if (GlobalDefaults.JOURNAL_ACTION_DEF) {
                                            director.journal("AccumulateData " + AccumulatingAction.this.getName() 
                                                    + ", id= " + AccumulatingAction.this.getID()
                                                    + ", generated " + sum);
                                    }

 
        private Delay accumulatorDelay= AccumulatorDefaults.ACCUMULATE_DELAY_DEF;

        Function out= new AccumulateData();
        Function inAck= new ActionFired();

        public void setDelay(Delay d) {
                accumulatorDelay= (Delay)d.clone();
                out.setDelay(accumulatorDelay);
        }

        private class AccumulateData extends Function {

                private int sum;

                public AccumulateData() {
                        super(AccumulatorDefaults.ACCUMULATE_DATA);
                        this.sum= 0;
                        functionDelay= accumulatorDelay;
                }

                public Object value() {
                        // get the input terminal to the Accumulator
                        Terminal t= AccumulatingAction.this.getTerminal(AccumulatorDefaults.IN);


                } // value
        } // AccumulateData

        private class ActionFired extends Function {

                public ActionFired() {
                        super("Vacous");
                        functionDelay= AccumulatorDefaults.ACK_DELAY_DEF;
                }

                public Object value() {
                        Iterator ti= terminals.iterator();
                        Terminal t= null;
                        boolean done= false;
                        while (!done && ti.hasNext()) {
                                t= (Terminal)(ti.next());
                                if (t.getBaseName().equals(AccumulatorDefaults.ACK_IN)) {
                                        done= true;
                                }
                        }
                        if (t != null ) {
                                director.journal("Accumulator received " + t.getData());
                        }
                        return null;
                } // value
        } // AcknowledgeData
*/
